﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Runtime.CompilerServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AsyncAwait
{
    public class ViewModel : INotifyPropertyChanged
    {
        private const string url = "http://www.bleedingedge.si/";
        private const string pattern = @"<a[^>]*?href=[""'](?<href>\b(https?)://[^\[\]""]+?)[""'][^>]*?>";

        private List<string> hyperlinks;
        public List<string> Hyperlinks
        {
            get { return hyperlinks; }
            set
            {
                if (hyperlinks == value)
                    return;
                hyperlinks = value;
                RaisePropertyChanged();
            }
        }

        public RelayCommand DownloadCommand { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged([CallerMemberName]string propertyName = null)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public ViewModel()
        {
            DownloadCommand = new RelayCommand(() => DownloadHyperlinksSynchronous());
            //DownloadCommand = new RelayCommand(() => DownloadHyperlinksAsync());
            //DownloadCommand = new RelayCommand(() => DownloadHyperlinksTaskAsync());
            //DownloadCommand = new RelayCommand(() => DownloadHyperlinksAsyncAwait());
        }

        private void DownloadHyperlinksSynchronous()
        {
            var client = new WebClient();
            string response = client.DownloadString(new Uri(url));
            var matches = Regex.Matches(response, pattern);

            Hyperlinks = matches.Cast<Match>().Select(m => m.Groups["href"].Value).ToList();
        }

        private void DownloadHyperlinksAsync()
        {
            var client = new WebClient();
            client.DownloadStringCompleted += (s, e) =>
                {
                    var matches = Regex.Matches(e.Result, pattern);

                    Hyperlinks = matches.Cast<Match>().Select(m => m.Groups["href"].Value).ToList();
                };

            client.DownloadStringAsync(new Uri(url));
        }

        private void DownloadHyperlinksTaskAsync()
        {
            var client = new WebClient();
            client.DownloadStringTaskAsync(new Uri(url)).ContinueWith(e =>
            {
                var matches = Regex.Matches(e.Result, pattern);

                Hyperlinks = matches.Cast<Match>().Select(m => m.Groups["href"].Value).ToList();
            });
        }

        private async void DownloadHyperlinksAsyncAwait()
        {
            var client = new WebClient();
            string response = await client.DownloadStringTaskAsync(new Uri(url));
            var matches = Regex.Matches(response, pattern);

            Hyperlinks = matches.Cast<Match>().Select(m => m.Groups["href"].Value).ToList();
        }
    }
}
