﻿using System;
using System.Activities;
using System.Activities.Expressions;
using System.Activities.Statements;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.CSharp.Activities;
using System.Windows.Media;
using System.Activities.XamlIntegration;
using System.Reflection;

namespace Expressions
{
    public class CsExpressionParser : IExpressionParser
    {
        public TResult Evaluate<TArg, TResult>(TArg input, string expression)
        {
            var cs = new CSharpValue<TResult>(expression ?? String.Empty);

            var wf = new DynamicActivity<TResult>
            {
                Name = "Temp",
                Properties =
                {
                    new DynamicActivityProperty
                    {
                        Name = "Input",
                        Type = typeof(InArgument<TArg>)
                    }
                },
                Implementation = () => new Assign<TResult>
                {
                    Value = new InArgument<TResult>(cs),
                    To = new ArgumentReference<TResult> { ArgumentName = "Result" }
                }
            };

            var inputs = new Dictionary<string, object>
            {
                { "Input", input }
            };

            typeof(ActivityXamlServices).GetMethod("Compile", BindingFlags.NonPublic | BindingFlags.Static).Invoke(null, new object[] { wf, new ActivityXamlServicesSettings().LocationReferenceEnvironment });

            return WorkflowInvoker.Invoke(wf, inputs);
        }

        public string Validate<TArg, TResult>(string expression)
        {
            var cs = new CSharpValue<TResult>(expression);

            var wf = new DynamicActivity<TResult>
            {
                Name = "Temp",
                Properties =
                {
                    new DynamicActivityProperty
                    {
                        Name = "Input",
                        Type = typeof(InArgument<TArg>)
                    }
                },
                Implementation = () => new If()
                {
                    Condition = new InArgument<bool>(false),
                    Then = new Assign<TResult>
                    {
                        Value = new InArgument<TResult>(cs),
                        To = new ArgumentReference<TResult> { ArgumentName = "Result" }
                    }
                }
            };

            try
            {
                typeof(ActivityXamlServices).GetMethod("Compile", BindingFlags.NonPublic | BindingFlags.Static).Invoke(null, new object[] { wf, new ActivityXamlServicesSettings().LocationReferenceEnvironment });
            }
            catch (TargetInvocationException e)
            {
                return e.InnerException.Message;
            }
            return null;
        }
    }
}
