﻿using System;
using System.Activities;
using System.Activities.Expressions;
using System.Activities.Statements;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualBasic.Activities;

namespace Expressions
{
    public class VbExpressionParser : IExpressionParser
    {
        public TResult Evaluate<TArg, TResult>(TArg input, string expression)
        {
            var vb = new VisualBasicValue<TResult>(expression);

            var wf = new DynamicActivity<TResult>
            {
                Properties =
                {
                    new DynamicActivityProperty
                    {
                        Name = "Input",
                        Type = typeof(InArgument<TArg>)
                    }
                },
                Implementation = () => new Assign<TResult>
                {
                    Value = new InArgument<TResult>(vb),
                    To = new ArgumentReference<TResult> { ArgumentName = "Result" }
                }
            };

            var inputs = new Dictionary<string, object>
            {
                { "Input", input }
            };

            return WorkflowInvoker.Invoke(wf, inputs);
        }

        public string Validate<TArg, TResult>(string expression)
        {
            var vb = new VisualBasicValue<TResult>(expression);

            var wf = new DynamicActivity<TResult>
            {
                Properties =
                {
                    new DynamicActivityProperty
                    {
                        Name = "Input",
                        Type = typeof(InArgument<TArg>)
                    }
                },
                Implementation = () => new If()
                {
                    Condition = new InArgument<bool>(false),
                    Then = new Assign<TResult>
                    {
                        Value = new InArgument<TResult>(vb),
                        To = new ArgumentReference<TResult> { ArgumentName = "Result" }
                    }
                }
            };

            try
            {
                WorkflowInvoker.Invoke(wf);
            }
            catch (InvalidWorkflowException e)
            {
                return e.Message;
            }
            return null;
        }
    }
}
