﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Expressions
{
    public class ViewModel : IDataErrorInfo, INotifyPropertyChanged
    {
        private readonly IExpressionParser expressionParser;

        public ViewModel(IExpressionParser expressionParser)
        {
            this.expressionParser = expressionParser;
            Expression = "Input.Length > 5";
            Value = String.Empty;
        }

        private string expression;
        public string Expression
        {
            get
            {
                return expression;
            }
            set
            {
                expression = value;
                RaisePropertyChanged("Expression");
                RaisePropertyChanged("Result");
            }
        }

        private string value;
        public string Value
        {
            get
            {
                return value;
            }
            set
            {
                this.value = value;
                RaisePropertyChanged("Value");
                RaisePropertyChanged("Result");
            }
        }

        public bool Result
        {
            get
            {
                try
                {
                    return expressionParser.Evaluate<string, bool>(value, expression);
                }
                catch
                {
                    return false;
                }
            }
        }

        public string Error
        {
            get { return null; }
        }

        public string this[string columnName]
        {
            get
            {
                switch (columnName)
                {
                    case "Expression":
                        if (!String.IsNullOrEmpty(expression))
                            return expressionParser.Validate<string, bool>(expression);
                        else
                            return null;
                    case "Value":
                        try
                        {
                            if (expression != null && expressionParser.Validate<string, bool>(expression) == null)
                                expressionParser.Evaluate<string, bool>(value, expression);
                            return null;
                        }
                        catch (Exception e)
                        {
                            return e.Message;
                        }
                    default:
                        return null;
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
