﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using SideBySideClient.OrderService;

namespace SideBySideClient
{
    public class ViewModel
    {
        private readonly IDialogService dialogService;

        public ObservableCollection<OrderInfo> Orders { get; set; }
        public OrderInfo SelectedOrder { get; set; }
        public string Customer { get; set; }

        public ICommand StartCommand { get; set; }
        public ICommand CompleteCommand { get; set; }

        public ViewModel(IDialogService dialogService)
        {
            this.dialogService = dialogService;

            Orders = new ObservableCollection<OrderInfo>();

            StartCommand = new RelayCommand(OnStart);
            CompleteCommand = new RelayCommand(OnComplete); 
        }

        private void OnStart(object obj)
        {
            try
            {
                if (String.IsNullOrEmpty(Customer))
                    return;

                var proxy = new OrderServiceClient();
                var orderInfo = new OrderInfo
                {
                    Customer = Customer,
                    OrderId = proxy.StartOrder(Customer)
                };

                Orders.Add(orderInfo);

                dialogService.ShowMsgBox(String.Format("Order ({0}) started for {1}.", orderInfo.OrderId, orderInfo.Customer));
            }
            catch (Exception e)
            {
                dialogService.ShowMsgBox(e.Message);
            }
        }

        private void OnComplete(object obj)
        {
            try
            {
                if (SelectedOrder == null)
                    return;

                var proxy = new OrderServiceClient();
                var msg = proxy.CompleteOrder(SelectedOrder.OrderId);

                Orders.Remove(SelectedOrder);

                dialogService.ShowMsgBox(msg);

            }
            catch (Exception e)
            {
                dialogService.ShowMsgBox(e.Message);
            }
        }
    }

}
